@@ -35,14 +35,14 @@ pub trait Callback<P>: Send + Sync + 'static {
 pub trait Key: Eq + Hash + Clone + Send + Sync + Debug + 'static {}
 impl<K: Eq + Hash + Clone + Send + Sync + Debug + 'static> Key for K {}
 
-#[derive(Copy, Clone, Debug, PartialEq, Eq, serde::Deserialize)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, serde::Deserialize, serde::Serialize)]
 pub enum Action {
 	Insert,
 	Change,
 	Remove,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq, serde::Deserialize)] //deserialize is for integration tests
+#[derive(Clone, Debug, PartialEq, Eq, serde::Deserialize, serde::Serialize)] //deserialize is for integration tests, serialize if for wasm js binding
 pub struct Event<K: Key> {
 	pub key: K,
 	pub action: Action,
