@@ -7,18 +7,16 @@ At the moment the best documentation/reference is the [Ruby binding](https://git
 
 mod descriptor;
 mod local_node;
+mod logger;
 mod object_expose;
 mod object_observe;
-#[cfg(feature = "env_logger")]
-mod std_logger;
 mod tag_expose;
 mod tag_observe;
 
 pub use descriptor::*;
 pub use local_node::*;
+pub use logger::*;
 pub use object_expose::*;
 pub use object_observe::*;
-#[cfg(feature = "env_logger")]
-pub use std_logger::*;
 pub use tag_expose::*;
 pub use tag_observe::*;
