@@ -3,12 +3,11 @@
 //we'd rather have every unsafe block explicit, so disabling warning that function is not unsafe
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
 
-use wasm_bindgen::prelude::*;
+use log::{error, trace};
 
 use crate::LocalNode;
 
 
-#[wasm_bindgen]
 #[derive(Copy, Clone, Debug)]
 #[repr(u8)]
 pub enum FFIError {
@@ -21,25 +20,29 @@ pub enum FFIError {
 
 
 #[repr(C)]
-#[wasm_bindgen]
 pub struct FFILocalNodeNewResult {
 	pub error: FFIError,
-	pub local_node: *mut LocalNode,
+	pub local_node_pointer: *mut LocalNode,
 }
 
 //TODO: capacity, limit
+
 #[no_mangle]
 pub extern "C" fn hakuban_local_node_new(name_c: *const i8) -> FFILocalNodeNewResult {
 	let name_str: &str = match unsafe { std::ffi::CStr::from_ptr(name_c).to_str() } {
 		Ok(string) => string,
-		Err(_error) => {
-			return FFILocalNodeNewResult { error: FFIError::InvalidString, local_node: std::ptr::null_mut() };
+		Err(error) => {
+			error!("Couldn't convert node name parameter to string: {:?}", error);
+			return FFILocalNodeNewResult { error: FFIError::InvalidString, local_node_pointer: std::ptr::null_mut() };
 		}
 	};
-	FFILocalNodeNewResult { error: FFIError::None, local_node: Box::into_raw(Box::new(LocalNode::builder().with_name(name_str).build())) }
+	trace!("Constructing local node {:?}", name_str);
+	FFILocalNodeNewResult { error: FFIError::None, local_node_pointer: Box::into_raw(Box::new(LocalNode::builder().with_name(name_str).build())) }
 }
 
 #[no_mangle]
 pub extern "C" fn hakuban_local_node_drop(local_node: *mut LocalNode) {
-	drop(unsafe { Box::from_raw(local_node) });
+	let local_node = unsafe { Box::from_raw(local_node) };
+	trace!("Dropping local node {:?}", local_node);
+	drop(local_node);
 }
