@@ -3,7 +3,7 @@ use std::{collections::{HashMap, VecDeque}, sync::{atomic::Ordering, Arc, Mutex}
 use log::trace;
 
 use super::{core::NodeCore, local::LocalNodeShared};
-use crate::{diff, events::{Action, Event, EventEmitter, EventMap, EventMerge, EventMergeInput}, message::{Change, DataChange, Message, ObjectChange, OptionsChange, TagChange}, object::{core::{ObjectRawData, ObjectVersion}, remote::RemoteObject}, tag::remote::RemoteTag, LocalNode, ObjectDescriptor, ObjectType, TagDescriptor};
+use crate::{diff, events::{Action, Event, EventEmitter, EventMap, EventMerge, EventMergeInput}, message::{Change, DataChange, Message, ObjectChange, OptionsChange, TagChange}, object::{core::{ObjectRawData, ObjectVersion}, remote::RemoteObject}, tag::remote::RemoteTag, Descriptor, LocalNode, ObjectDescriptor, ObjectType, TagDescriptor};
 
 /*
 This file is a bit heavy with data structures, because there is a lot to keep track of.
@@ -30,8 +30,8 @@ pub(super) struct OptionsState {
 }
 
 pub(super) struct ObjectState {
-	pub(super) received: RemoteObjectState,
-	pub(super) sent: RemoteObjectState,
+	pub(super) received: ReceivedRemoteObjectState,
+	pub(super) sent: SentRemoteObjectState,
 	pub(super) local: LocalObjectState,
 }
 
@@ -41,13 +41,35 @@ pub(super) struct TagState {
 	pub(super) local: LocalTagState,
 }
 
-pub(super) struct RemoteObjectState {
+#[derive(Debug)]
+struct ArchivedSentDataVersion {
+	can_be_dropped_at_ack: u64, // only used for dropping archived versions from "sent" archive
+	data_version: ObjectVersion,
+	data_transmitted: bool, // used only for "sent"
+	data_type: ObjectType,
+	data: ObjectRawData,
+}
+
+#[derive(Debug)]
+struct ArchivedReceivedDataVersion {
+	data_version: ObjectVersion,
+	data_type: Option<ObjectType>, // None if data_version was only declared by sender, without sending data
+	data: Option<ObjectRawData>,   // None if remote side didn't send data and we don't know the data version submitted
+}
+
+
+pub(super) struct SentRemoteObjectState {
 	pub(super) observed: bool,
 	pub(super) exposed: bool,
-	pub(super) data: VecDeque<(/*can drop on ack*/ u64, /*version*/ ObjectVersion, /* data sent */ bool, /*data*/ ObjectRawData)>,
-	pub(super) data_type: Option<ObjectType>,
-	//TODO: is this redundant?
-	pub(super) data_version: Option<ObjectVersion>,
+	data: VecDeque<ArchivedSentDataVersion>, // newest versions on the front
+	//TODO: this doesn't work
+	pub(super) cost: Option<Option<u32>>,
+}
+
+pub(super) struct ReceivedRemoteObjectState {
+	pub(super) observed: bool,
+	pub(super) exposed: bool,
+	data: VecDeque<ArchivedReceivedDataVersion>, // newest versions on the front
 	//TODO: this doesn't work
 	pub(super) cost: Option<Option<u32>>,
 }
@@ -83,8 +105,8 @@ impl State {
 	pub(super) fn object_borrow_mut(&mut self, descriptor: &ObjectDescriptor, local_node: &Arc<LocalNodeShared>, node: &Arc<NodeCore>) -> &mut ObjectState {
 		if !self.objects.contains_key(descriptor) {
 			let object_state = ObjectState {
-				received: RemoteObjectState { observed: false, exposed: false, data: VecDeque::new(), data_type: None, data_version: None, cost: None },
-				sent: RemoteObjectState { observed: false, exposed: false, data: VecDeque::new(), data_type: None, data_version: None, cost: None },
+				received: ReceivedRemoteObjectState { observed: false, exposed: false, data: VecDeque::new(), cost: None },
+				sent: SentRemoteObjectState { observed: false, exposed: false, data: VecDeque::new(), cost: None },
 				local: LocalObjectState { object_interface: RemoteObject::new(local_node.clone(), node.clone(), descriptor), changes_token: None },
 			};
 			self.objects.insert(descriptor.clone(), object_state);
@@ -229,12 +251,10 @@ impl State {
 }
 
 
-impl RemoteObjectState {
+impl SentRemoteObjectState {
 	pub(super) fn apply_change(&mut self, change: &ObjectChange) {
 		self.observed = change.observer.unwrap_or(self.observed);
 		self.exposed = change.exposer.unwrap_or(self.exposed);
-		self.data_version = change.data_version.clone().or_else(|| self.data_version.take()); //because I hate "if"
-		self.data_type = change.data_type.clone().or_else(|| self.data_type.take());
 		self.cost = change.cost.or(self.cost);
 	}
 }
@@ -247,11 +267,6 @@ impl RemoteTagState {
 	}
 }
 
-#[derive(Clone, Eq, PartialEq, Hash, Debug)]
-pub enum Descriptor {
-	Object(ObjectDescriptor),
-	Tag(TagDescriptor),
-}
 
 pub struct RemoteNodeMutable {
 	state: State,
@@ -372,7 +387,7 @@ impl RemoteNode {
 
 	//TODO replace String error with proper error enum
 	pub fn received_message(&self, message: Message) -> Result<Vec<Message>, String> {
-		trace!("Message received:\n{}", message.inspect());
+		trace!("{:?} message received\n{}", self.node_core, message.inspect());
 
 		let changes_to_send = vec![];
 		let mut mutable = self.mutable.lock().unwrap();
@@ -419,25 +434,34 @@ impl RemoteNode {
 						object_state.received.exposed = exposer;
 					}
 					if let Some(data_version) = &object_change.data_version {
+						//TODO: fail if same or lower data_version received?
+						//TODO: fail if data_type received without data
 						if let Some(data) = object_change.data {
 							match data {
 								DataChange::Data { data } => {
-									object_state.received.data.clear();
-									object_state.received.data.push_back((message.sequence_number, data_version.clone(), true, data));
+									let previous_archived_data = object_state.received.data.pop_back();
+									object_state.received.data.push_front(ArchivedReceivedDataVersion {
+										data_type: Some(object_change.data_type.or_else(|| previous_archived_data.unwrap().data_type).unwrap()),
+										data_version: data_version.clone(),
+										data: Some(data),
+									});
 								}
 								DataChange::Diff { diff, old_data_version } => {
+									let previous_archived_data = object_state.received.data.pop_back();
 									let empty = Arc::new(vec![]);
-									let old_data = if let Some(old_data_version) = old_data_version {
-										if let Some((_sequence, _version, _data_present, data)) =
-											object_state.sent.data.iter().find(|(_sequence, version, _data_present, _data)| *version == old_data_version)
+									let (old_data, old_data_type) = if let Some(old_data_version) = old_data_version {
+										if let Some(archived_data) =
+											object_state.sent.data.iter().find(|archived_data| *archived_data.data_version == old_data_version)
 										{
-											data.clone()
-										} else if let Some((_sequence, version, _data_present, data)) = object_state.received.data.pop_back() {
-											if *version == old_data_version {
-												data
-											} else {
-												return Err("CAKE not detected in receptacle(1)".to_string());
-											}
+											(&archived_data.data, Some(&archived_data.data_type))
+										} else if previous_archived_data.is_some()
+											&& previous_archived_data.as_ref().unwrap().data.is_some()
+											&& *previous_archived_data.as_ref().unwrap().data_version == old_data_version
+										{
+											(
+												previous_archived_data.as_ref().unwrap().data.as_ref().unwrap(),
+												Some(previous_archived_data.as_ref().unwrap().data_type.as_ref().unwrap()),
+											)
 										} else {
 											//this can happen if remote side sent us data while we lost interest in the object
 											//TEST-NEEDED
@@ -445,28 +469,21 @@ impl RemoteNode {
 											continue;
 										}
 									} else {
-										empty
+										(&empty, None)
 									};
-									object_state.received.data.clear();
-									object_state.received.data.push_back((
-										message.sequence_number,
-										data_version.clone(),
-										true,
-										Arc::new(diff::patch(&old_data, &diff)?),
-									));
+									object_state.received.data.push_front(ArchivedReceivedDataVersion {
+										data_version: data_version.clone(),
+										data: Some(Arc::new(diff::patch(old_data, &diff)?)),
+										data_type: Some(object_change.data_type.or_else(|| old_data_type.cloned()).unwrap()),
+									});
 								}
-							}
-						};
-						if let Some(data_type) = object_change.data_type {
-							object_state.received.data_type = Some(data_type);
-						};
-						if !object_state.received.data.is_empty() && object_state.received.data_type.is_some() {
+							};
 							if object_state.local.object_interface.is_exposed_by_this_node() {
-								//TODO: or should we check if node is active exposer?
+								//TODO: or should we check if node is the **assigned** exposer?
 								object_state.local.object_interface.state_set(
 									data_version,
-									object_state.received.data.back().as_ref().unwrap().3.clone(),
-									object_state.received.data_type.as_ref().unwrap().clone(),
+									object_state.received.data.front().as_ref().unwrap().data.as_ref().unwrap().clone(),
+									object_state.received.data.front().as_ref().unwrap().data_type.as_ref().unwrap().clone(),
 								);
 							} else {
 								return Err("Remote state error. Data submitted for object remote is not a exposer of.".to_string());
@@ -474,8 +491,13 @@ impl RemoteNode {
 						} else {
 							//TODO: if there is no data sent, should that only be permitted for initial message? only for observers?
 							object_state.local.object_interface.maybe_synchronize(data_version);
+							let _previous_archived_data = object_state.received.data.pop_back();
+							object_state.received.data.push_front(ArchivedReceivedDataVersion {
+								data_version: data_version.clone(),
+								data_type: None,
+								data: None, //TODO: lookup locally
+							});
 						}
-						object_state.received.data_version = object_change.data_version;
 					} else if object_change.data.is_some() || object_change.data_type.is_some() {
 						return Err("Remote state error. Data or data type submitted without data version.".to_string());
 					}
@@ -505,8 +527,8 @@ impl RemoteNode {
 		}
 
 		for object_state in mutable.state.objects.values_mut() {
-			while object_state.sent.data.len() > 1 && object_state.sent.data.front().unwrap().0 <= message.acknowledgment_number {
-				object_state.sent.data.pop_front();
+			while object_state.sent.data.len() > 1 && object_state.sent.data.front().unwrap().can_be_dropped_at_ack <= message.acknowledgment_number {
+				object_state.sent.data.pop_back();
 			}
 		}
 
@@ -528,15 +550,18 @@ impl RemoteNode {
 		} else {
 			mutable.last_sent_message_sequence_number += 1;
 			mutable.last_sent_message_acknowledgement_number = mutable.last_received_message_sequence_number;
-			vec![Message {
+			let ret = vec![Message {
 				sequence_number: mutable.last_sent_message_sequence_number,
 				acknowledgment_number: mutable.last_received_message_sequence_number,
 				changes,
-			}]
+			}];
+			trace!("{:?}, message sending:\n{}", self.node_core, ret[0].inspect());
+			ret
 		}
 	}
 
 	fn process_object_event(&self, mutable: &mut RemoteNodeMutable, descriptor: ObjectDescriptor, action: Action) -> Result<Vec<Change>, String> {
+		trace!("{:?}, object event received {:?} {:?}", self.node_core, &action, &descriptor);
 		let mut changes = vec![];
 		match action {
 			Action::Insert | Action::Change => {
@@ -544,9 +569,15 @@ impl RemoteNode {
 					let options = mutable.state.options.clone();
 					let object_state = mutable.state.object_borrow_mut(&descriptor, &self.local_node, &self.node_core);
 
-					let last_received_version = object_state.received.data_version.clone();
-					let last_sent_version = object_state.sent.data_version.clone();
-					let highest_version_known_by_remote = vec![last_received_version.as_ref(), last_sent_version.as_ref()].into_iter().flatten().max();
+					let last_received_version = object_state.received.data.front();
+					let last_sent_version = object_state.sent.data.iter().find(|archived_version| archived_version.data_transmitted);
+					let highest_version_known_by_remote = vec![
+						last_received_version.as_ref().map(|archived| &archived.data_version),
+						last_sent_version.as_ref().map(|archived| &archived.data_version),
+					]
+					.into_iter()
+					.flatten()
+					.max();
 
 					let (_synchronized, data_version, data_type, data) = object_state.local.object_interface.state();
 					let local_node_knows_newer_version_than_remote = data_version
@@ -569,66 +600,63 @@ impl RemoteNode {
 
 					let declare_cost = if declare_local_as_exposer { object_state.local.object_interface.minimum_cost_in_other_nodes() } else { None };
 
-					let declare_data = if object_state.local.object_interface.is_assigned_to_other_node()
+					let (declare_data, data_type_of_declared_data) = if object_state.local.object_interface.is_assigned_to_other_node()
 						&& object_state.local.object_interface.is_observed_by_this_node()
 						&& local_node_knows_newer_version_than_remote
 					{
 						if mutable.diff_produce.unwrap_or_else(|| options.diff_request.unwrap_or(true)) {
-							//TODO: this is messy af
-							let (newest_shared_version, newest_shared_data) = match (last_received_version, last_sent_version) {
-								(None, None) => (None, None),
+							let (newest_shared_data, newest_shared_data_type, newest_shared_data_version) = match (last_received_version, last_sent_version) {
+								(None, None) => (None, None, None),
 								(Some(received_version), None) =>
-									if object_state.received.data.back().is_some() {
-										(Some(received_version), object_state.received.data.back())
+									if received_version.data.is_some() {
+										(
+											Some(received_version.data.as_ref().unwrap()),
+											Some(received_version.data_type.as_ref().unwrap()),
+											Some(&received_version.data_version),
+										)
 									} else {
-										(None, None)
-									},
-								(None, Some(sent_version)) =>
-									if let Some(sent) = object_state.sent.data.back() {
-										if sent.2 {
-											(Some(sent_version), Some(sent))
-										} else {
-											(None, None)
-										}
-									} else {
-										(None, None)
+										(None, None, None)
 									},
+								(None, Some(sent_version)) => (Some(&sent_version.data), Some(&sent_version.data_type), Some(&sent_version.data_version)),
 								(Some(received_version), Some(sent_version)) =>
-									if received_version > sent_version && object_state.received.data.back().is_some() {
-										(Some(received_version), object_state.received.data.back())
-									} else if let Some(sent) = object_state.sent.data.back() {
-										if sent.2 {
-											(Some(sent_version), Some(sent))
-										} else {
-											(None, None)
-										}
+									if received_version.data_version > sent_version.data_version && received_version.data.is_some() {
+										(
+											Some(received_version.data.as_ref().unwrap()),
+											Some(received_version.data_type.as_ref().unwrap()),
+											Some(&received_version.data_version),
+										)
 									} else {
-										(None, None)
+										(Some(&sent_version.data), Some(&sent_version.data_type), Some(&sent_version.data_version))
 									},
 							};
 							let diff = object_state.local.object_interface.get_diff(
-								&newest_shared_version,
-								newest_shared_data.map(|data| &data.3),
+								newest_shared_data_version,
+								newest_shared_data,
 								data_version.as_ref().unwrap(),
 								data.as_ref().unwrap(),
 							);
 							if let Some(diff) = diff {
-								Some(DataChange::Diff { diff, old_data_version: newest_shared_version })
+								(
+									Some(DataChange::Diff { diff, old_data_version: newest_shared_data_version.cloned() }),
+									newest_shared_data_type.or_else(|| Some(data_type.as_ref().unwrap())),
+								)
 							} else {
-								Some(DataChange::Data { data: data.clone().unwrap() })
+								(Some(DataChange::Data { data: data.clone().unwrap() }), Some(data_type.as_ref().unwrap()))
 							}
 						} else {
-							Some(DataChange::Data { data: data.clone().unwrap() })
+							(Some(DataChange::Data { data: data.clone().unwrap() }), Some(data_type.as_ref().unwrap()))
 						}
 					} else {
-						None
+						(None, None)
 					};
 
+					let declare_data_type = data_type_of_declared_data
+						.filter(|new_data_type| last_sent_version.map(|last_sent_version| last_sent_version.data_type != **new_data_type).unwrap_or(true));
+
 					let change = ObjectChange {
 						descriptor: descriptor.clone(),
 						observer: if declare_local_as_observer != object_state.sent.observed { Some(declare_local_as_observer) } else { None },
 						exposer: if declare_local_as_exposer != object_state.sent.exposed { Some(declare_local_as_exposer) } else { None },
-						data_type: if declare_data.is_some() && data_type != object_state.sent.data_type { Some(data_type.unwrap()) } else { None },
 						data_version: if declare_data.is_some()
 							|| ((declare_local_as_observer && !object_state.sent.observed) || (declare_local_as_exposer && !object_state.sent.exposed))
 						{
@@ -636,6 +664,7 @@ impl RemoteNode {
 						} else {
 							None
 						},
+						data_type: declare_data_type.cloned(),
 						data: declare_data,
 						cost: if Some(declare_cost) != object_state.sent.cost { Some(declare_cost) } else { None },
 					};
@@ -643,12 +672,13 @@ impl RemoteNode {
 
 					if !change.is_empty() {
 						if change.data_version.is_some() {
-							object_state.sent.data.push_back((
-								mutable.last_sent_message_sequence_number,
-								data_version.unwrap(),
-								change.data.is_some(),
-								data.unwrap(),
-							));
+							object_state.sent.data.push_front(ArchivedSentDataVersion {
+								can_be_dropped_at_ack: mutable.last_sent_message_sequence_number,
+								data_version: data_version.unwrap(),
+								data_transmitted: change.data.is_some(),
+								data_type: data_type.unwrap(),
+								data: data.unwrap(),
+							});
 						}
 						changes.push(Change::Object(change));
 					};
