@@ -9,7 +9,7 @@ use log::{error, info, trace};
 use tokio::{net::TcpListener, time::{sleep, timeout}};
 use url::Url;
 
-use crate::{events::EventStream, message::Message, node::remote::{Descriptor, RemoteNode}, LocalNode};
+use crate::{events::EventStream, message::Message, node::remote::RemoteNode, Descriptor, LocalNode};
 
 
 // tiny state machine engine
@@ -594,23 +594,27 @@ where
 				}
 			};
 			// trying to receive something
-			if let Ok(message) = timeout(min(self.parameters.timeout / 4, Duration::from_micros(1000)), ws_stream_read.as_mut().unwrap().next()).await {
+			if let Ok(message) = timeout(min(self.parameters.timeout / 4, Duration::from_millis(1000)), ws_stream_read.as_mut().unwrap().next()).await {
 				match message {
 					Some(Ok(tungstenite::Message::Binary(message_msgpack))) => {
 						*self.last_received_at.lock().unwrap() = Instant::now();
-						return rmp_serde::from_read_ref::<Vec<u8>, Message>(&message_msgpack).map_err(Termination::from_other_error);
+						if !message_msgpack.is_empty() {
+							return rmp_serde::from_read_ref::<Vec<u8>, Message>(&message_msgpack).map_err(Termination::from_other_error);
+						}
 					}
 					Some(Ok(tungstenite::Message::Ping(_payload))) => {
+						//trace!("Received ping");
 						*self.last_received_at.lock().unwrap() = Instant::now();
 					}
 					Some(Ok(tungstenite::Message::Pong(_payload))) => {
+						//trace!("Received pong");
 						*self.last_received_at.lock().unwrap() = Instant::now();
 					}
 					Some(Ok(tungstenite::Message::Close(_payload))) => {
 						return Err(Termination::SocketClosed);
 					}
-					Some(Ok(_other_message)) => {
-						todo!();
+					Some(Ok(other_message)) => {
+						todo!("Got unexpected message: {:?}", other_message);
 					}
 					Some(Err(error)) => return Err(Termination::from_tungstenite_error(error)),
 					None => todo!(),
