@@ -112,19 +112,22 @@ impl RealClient {
 											tokio::select! {
 												next_message = stream.next() => match next_message {
 													Some(Ok(tungstenite::Message::Binary(message_msgpack))) => {
-														let target_changes = rmp_serde::from_read_ref::<Vec<u8>, Message>(&message_msgpack).unwrap().changes;
+														let received_changes = rmp_serde::from_read_ref::<Vec<u8>, Message>(&message_msgpack).unwrap().changes;
 														let found_at = expected_messages
 															.iter()
 															.enumerate()
 															.find(|(_, expected_message)| {
-																for (expected_change, target_change) in expected_message.changes.iter().zip(target_changes.iter()) {
+																if expected_message.changes.len() != received_changes.len() {
+																	return false
+																}
+																for (expected_change, target_change) in expected_message.changes.iter().zip(received_changes.iter()) {
 																	if expected_change != target_change {
 																		return false;
 																	};
 																}
 																true
 															})
-															.unwrap_or_else(|| panic!("❌ERROR: {} received message different than expected.\nExpected:\n{:?}\nReceived:\n{:?}\n", name, expected_messages, target_changes))
+															.unwrap_or_else(|| panic!("❌ERROR: {} received message different than expected.\nExpected:\n{:?}\nReceived:\n{:?}\n", name, expected_messages, received_changes))
 															.0;
 														expected_messages.remove(found_at);
 														to_send.push(message_msgpack);
